@@ -28,6 +28,8 @@ local regexp = require "rspamd_regexp"
 local rspamd_expression = require "rspamd_expression"
 local rspamd_ip = require "rspamd_ip"
 local lua_util = require "lua_util"
+local rspamd_dns = require "rspamd_dns"
+local lua_selectors = require "lua_selectors"
 local redis_params
 local fun = require "fun"
 local N = 'multimap'
@@ -80,6 +82,9 @@ local value_types = {
   mempool = {
     get_value = function(val) return val end,
   },
+  selector = {
+    get_value = function(val) return val end,
+  },
   symbol_options = {
     get_value = function(val) return val end,
   },
@@ -93,6 +98,9 @@ local function apply_hostname_filter(task, filter, hostname, r)
   if filter == 'tld' then
     local tld = util.get_tld(hostname)
     return tld
+  elseif filter == 'top' then
+    local tld = util.get_tld(hostname)
+    return tld:match('[^.]*$') or tld
   else
     if not r['re_filter'] then
       local pat = string.match(filter, 'tld:regexp:(.+)')
@@ -123,6 +131,9 @@ local function apply_url_filter(task, filter, url, r)
 
   if filter == 'tld' then
     return url:get_tld()
+  elseif filter == 'top' then
+    local tld = url:get_tld()
+    return tld:match('[^.]*$') or tld
   elseif filter == 'full' then
     return url:get_text()
   elseif filter == 'is_phished' then
@@ -361,6 +372,7 @@ local multimap_filters = {
   url = apply_url_filter,
   filename = apply_filename_filter,
   mempool = apply_regexp_filter,
+  selector = apply_regexp_filter,
   hostname = apply_hostname_filter,
   --content = apply_content_filter, -- Content filters are special :(
 }
@@ -407,6 +419,7 @@ local function multimap_callback(task, rule)
             table.insert(srch, nip)
         end
       end
+
       table.insert(srch, 1, r['redis_key'])
       ret = rspamd_redis_make_request(task,
         redis_params, -- connect params
@@ -425,8 +438,17 @@ local function multimap_callback(task, rule)
     end
 
     if ret then
-      callback(ret)
+      if type(ret) == 'table' then
+        for _,elt in ipairs(ret) do
+          callback(elt)
+        end
+
+        ret = true
+      else
+        callback(ret)
+      end
     end
+
     return ret
   end
 
@@ -533,9 +555,9 @@ local function multimap_callback(task, rule)
             r['message'] = r.message_func(task, r['symbol'], opt)
           end
           if r['message'] then
-            task:set_pre_result(r['action'], r['message'])
+            task:set_pre_result(r['action'], r['message'], N)
           else
-            task:set_pre_result(r['action'], 'Matched map: ' .. r['symbol'])
+            task:set_pre_result(r['action'], 'Matched map: ' .. r['symbol'], N)
           end
         end
       end
@@ -678,10 +700,10 @@ local function multimap_callback(task, rule)
     local res,trace = rule['expression']:process_traced(task)
 
     if not res or res == 0 then
-      rspamd_logger.debugm(N, task, 'condition is false for %s', rule['symbol'])
+      lua_util.debugm(N, task, 'condition is false for %s', rule['symbol'])
       return
     else
-      rspamd_logger.debugm(N, task, 'condition is true for %s: %s', rule['symbol'],
+      lua_util.debugm(N, task, 'condition is true for %s: %s', rule['symbol'],
         trace)
     end
   end
@@ -694,24 +716,25 @@ local function multimap_callback(task, rule)
         if rt == 'ip' then
           match_rule(rule, ip)
         else
-          local cb = function (_, to_resolve, results, err)
-            task:inc_dns_req()
-            if err and (err ~= 'requested record is not found' and err ~= 'no records with this name') then
-              rspamd_logger.errx(task, 'error looking up %s: %s', to_resolve, err)
-            end
-            if results then
-              task:insert_result(rule['symbol'], 1, rule['map'])
+          local to_resolve = ip_to_rbl(ip, rule['map'])
 
-              if pre_filter then
-                task:set_pre_result(rule['action'], 'Matched map: ' .. rule['symbol'])
-              end
+          local is_ok, results = rspamd_dns.request({
+            type = "a",
+            task = task,
+            name = to_resolve,
+          })
+
+          lua_util.debugm(N, rspamd_config, 'resolve() finished: results=%1, is_ok=%2, to_resolve=%3', results, is_ok, to_resolve)
+
+          if not is_ok and (results ~= 'requested record is not found' and results ~= 'no records with this name') then
+            rspamd_logger.errx(task, 'error looking up %s: %s', to_resolve, results)
+          elseif is_ok then
+            task:insert_result(rule['symbol'], 1, rule['map'])
+            if pre_filter then
+              task:set_pre_result(rule['action'], 'Matched map: ' .. rule['symbol'], N)
             end
           end
 
-          task:get_resolver():resolve_a({task = task,
-            name = ip_to_rbl(ip, rule['map']),
-            callback = cb,
-          })
         end
       end
     end,
@@ -753,6 +776,7 @@ local function multimap_callback(task, rule)
     url = function()
       if task:has_urls() then
         local msg_urls = task:get_urls()
+
         for _,url in ipairs(msg_urls) do
           match_url(rule, url)
         end
@@ -823,7 +847,21 @@ local function multimap_callback(task, rule)
         end
       end
     end,
+    selector = function()
+      local elts = rule.selector(task)
+
+      if elts then
+        if type(elts) == 'table' then
+          for _,elt in ipairs(elts) do
+            match_rule(rule, elt)
+          end
+        else
+          match_rule(rule, elts)
+        end
+      end
+    end,
   }
+
   process_rule_funcs.ip = process_rule_funcs.dnsbl
   local f = process_rule_funcs[rt]
   if f then
@@ -841,6 +879,45 @@ end
 
 local function add_multimap_rule(key, newrule)
   local ret = false
+
+  local function multimap_load_hash(rule)
+    if rule['regexp'] then
+      if rule['multi'] then
+        rule['hash'] = rspamd_config:add_map ({
+          url = rule['map'],
+          description = rule['description'],
+          type = 'regexp_multi'
+        })
+      else
+        rule['hash'] = rspamd_config:add_map ({
+          url = newrule['map'],
+          description = newrule['description'],
+          type = 'regexp'
+        })
+      end
+    elseif rule['glob'] then
+      if rule['multi'] then
+        rule['hash'] = rspamd_config:add_map ({
+          url = rule['map'],
+          description = rule['description'],
+          type = 'glob_multi'
+        })
+      else
+        rule['hash'] = rspamd_config:add_map ({
+          url = rule['map'],
+          description = rule['description'],
+          type = 'glob'
+        })
+      end
+    else
+      rule['hash'] = rspamd_config:add_map ({
+        url = rule['map'],
+        description = rule['description'],
+        type = 'hash'
+      })
+    end
+  end
+
   if newrule['message_func'] then
     newrule['message_func'] = assert(load(newrule['message_func']))()
   end
@@ -865,6 +942,23 @@ local function add_multimap_rule(key, newrule)
     rspamd_logger.errx(rspamd_config, 'mempool map requires variable')
     return nil
   end
+  if newrule['type'] == 'selector' then
+    if not newrule['selector'] then
+      rspamd_logger.errx(rspamd_config, 'selector map requires selector definition')
+      return nil
+    else
+      local selector = lua_selectors.create_selector_closure(
+          rspamd_config, newrule['selector'], newrule['delimiter'] or "")
+
+      if not selector then
+        rspamd_logger.errx(rspamd_config, 'selector map has invalid selector: "%s", symbol: %s',
+            newrule['selector'], newrule['symbol'])
+        return nil
+      end
+
+      newrule.selector = selector
+    end
+  end
   -- Check cdb flag
   if type(newrule['map']) == 'string' and string.find(newrule['map'], '^cdb://.*$') then
     newrule['cdb'] = cdb.create(newrule['map'])
@@ -889,8 +983,8 @@ local function add_multimap_rule(key, newrule)
   else
     if type(newrule['map']) == 'string' then
       local map = urls[newrule['map']]
-      if map and map['type'] == newrule['type']
-        and map['regexp'] == newrule['regexp'] then
+      if map and map['regexp'] == newrule['regexp'] and
+          map['glob'] == newrule['glob'] then
         if newrule['type'] == 'ip' then
           newrule['radix'] = map['map']
         else
@@ -943,25 +1037,8 @@ local function add_multimap_rule(key, newrule)
             ret = true
           end
         else
-          if newrule['regexp'] then
-            newrule['hash'] = rspamd_config:add_map ({
-              url = newrule['map'],
-              description = newrule['description'],
-              type = 'regexp'
-            })
-          elseif newrule['glob'] then
-            newrule['hash'] = rspamd_config:add_map ({
-              url = newrule['map'],
-              description = newrule['description'],
-              type = 'glob'
-            })
-          else
-            newrule['hash'] = rspamd_config:add_map ({
-              url = newrule['map'],
-              description = newrule['description'],
-              type = 'hash'
-            })
-          end
+          multimap_load_hash(newrule)
+
           if newrule['hash'] then
             ret = true
             if type(newrule['map']) == 'string' then
@@ -977,36 +1054,21 @@ local function add_multimap_rule(key, newrule)
           end
         end
       elseif newrule['type'] == 'header'
-        or newrule['type'] == 'rcpt'
-        or newrule['type'] == 'from'
-        or newrule['type'] == 'helo'
-        or newrule['type'] == 'symbol_options'
-        or newrule['type'] == 'filename'
-        or newrule['type'] == 'url'
-        or newrule['type'] == 'content'
-        or newrule['type'] == 'hostname'
-        or newrule['type'] == 'asn'
-        or newrule['type'] == 'country'
-        or newrule['type'] == 'mempool' then
-        if newrule['regexp'] then
-          newrule['hash'] = rspamd_config:add_map ({
-            url = newrule['map'],
-            description = newrule['description'],
-            type = 'regexp'
-          })
-        elseif newrule['glob'] then
-          newrule['hash'] = rspamd_config:add_map ({
-            url = newrule['map'],
-            description = newrule['description'],
-            type = 'glob'
-          })
-        else
-          newrule['hash'] = rspamd_config:add_map ({
-            url = newrule['map'],
-            description = newrule['description'],
-            type = 'hash'
-          })
-        end
+          or newrule['type'] == 'rcpt'
+          or newrule['type'] == 'from'
+          or newrule['type'] == 'helo'
+          or newrule['type'] == 'symbol_options'
+          or newrule['type'] == 'filename'
+          or newrule['type'] == 'url'
+          or newrule['type'] == 'content'
+          or newrule['type'] == 'hostname'
+          or newrule['type'] == 'asn'
+          or newrule['type'] == 'country'
+          or newrule['type'] == 'mempool'
+          or newrule['type'] == 'selector'then
+
+        multimap_load_hash(newrule)
+
         if newrule['hash'] then
           ret = true
           if type(newrule['map']) == 'string' then
@@ -1052,7 +1114,7 @@ local function add_multimap_rule(key, newrule)
 
       local function process_atom(atom, task)
         local f_ret = task:has_symbol(atom)
-        rspamd_logger.debugm(N, rspamd_config, 'check for symbol %s: %s', atom, f_ret)
+        lua_util.debugm(N, rspamd_config, 'check for symbol %s: %s', atom, f_ret)
 
         if f_ret then
           return 1
@@ -1067,7 +1129,7 @@ local function add_multimap_rule(key, newrule)
         newrule['expression'] = expression
 
         fun.each(function(v)
-          rspamd_logger.debugm(N, rspamd_config, 'add dependency %s -> %s',
+          lua_util.debugm(N, rspamd_config, 'add dependency %s -> %s',
             newrule['symbol'], v)
           rspamd_config:register_dependency(newrule['symbol'], v)
         end, atoms)
@@ -1089,6 +1151,8 @@ if opts and type(opts) == 'table' then
       if not rule then
         rspamd_logger.errx(rspamd_config, 'cannot add rule: "'..k..'"')
       else
+        rspamd_logger.infox(rspamd_config, 'added multimap rule: %s (%s)',
+            k, rule.type)
         table.insert(rules, rule)
       end
     end
@@ -1114,20 +1178,11 @@ if opts and type(opts) == 'table' then
     end
     if rule['score'] then
       -- Register metric symbol
-      local description = 'multimap symbol'
-      local group = N
-      if rule['description'] then
-        description = rule['description']
-      end
-      if rule['group'] then
-        group = rule['group']
-      end
-      rspamd_config:set_metric_symbol({
-          name = rule['symbol'],
-          score = rule['score'],
-          description = description,
-          group = group
-      })
+      rule.name = rule.symbol
+      rule.description = rule.description or 'multimap symbol'
+      rule.group = rule.group or N
+
+      rspamd_config:set_metric_symbol(rule)
     end
   end,
   fun.filter(function(r) return not r['prefilter'] end, rules))
